#include "SVHelper.h"


float SVHelper::vectorExtend = 10.;

// is a face 'visible' from the light source
bool SVHelper::isVisibleToLight(int a, int b, int c, float pnts[], float lightpos[3])
{
	if(c < 0)
		return false;

	// calc the faceNormal for the given tri
	float tmp1x, tmp1y, tmp1z, tmp2x, tmp2y, tmp2z;

	// vec1
	tmp1x = pnts[b*3]   - pnts[a*3];
	tmp1y = pnts[b*3+1] - pnts[a*3+1];
	tmp1z = pnts[b*3+2] - pnts[a*3+2];

	// vec2
	tmp2x = pnts[c*3]   - pnts[a*3];
	tmp2y = pnts[c*3+1] - pnts[a*3+1];
	tmp2z = pnts[c*3+2] - pnts[a*3+2];

	float nx, ny, nz;
	// calc the normal as crossproduct of vec1 and vec2
	nx = tmp1y * tmp2z - tmp1z * tmp2y;
    ny = tmp1z * tmp2x - tmp1x * tmp2z;
	nz = tmp1x * tmp2y - tmp1y * tmp2x;

	// vec from light to tri
	tmp1x = lightpos[0] - pnts[c*3];
	tmp1y = lightpos[1] - pnts[c*3+1];
	tmp1z = lightpos[2] - pnts[c*3+2];

	// is the tri facing the light ???
	if( (nx * tmp1x + ny * tmp1y + nz * tmp1z) > 0)
		return true;
	return false;
}

bool SVHelper::isContourEdge(Edge* edge, float pnts[], float lightpos[3], bool &faceA)
{
	int a = edge->pointA;
	int b = edge->pointB;
	int c = edge->faceApoint;
	int d = edge->faceBpoint;

	faceA = isVisibleToLight(a, b, c, pnts, lightpos);
	bool faceB = isVisibleToLight(a, d, b, pnts, lightpos);

	// XOR - contour if one is lit and the other isnt
	return faceA ^ faceB;
}

void SVHelper::stretchEdge(float a[3], float b[3], float lightpos[3])
{
	// stretch the two points to a quad
	float tmpA[3], tmpB[3];
	// be sure that vectorExtend is big enough so the quad is stretched far enough
	// if the lightsource is too close to the geometry we need to normalize the vector first
	for(int i=0; i<3; i++)
	{
		tmpA[i] = a[i] + (a[i]-lightpos[i]) * vectorExtend;
		tmpB[i] = b[i] + (b[i]-lightpos[i]) * vectorExtend;
	}
	glVertex3fv(a);
	glVertex3fv(tmpA);
	glVertex3fv(tmpB);
	glVertex3fv(b);
}
